#include <Arduino_FreeRTOS.h>
#include <semphr.h>  // add the FreeRTOS functions for Semaphores (or Flags).

const int Trigger = 6;   //Pin digital 2 para el Trigger del sensor
const int Echo = 7;   //Pin digital 3 para el Echo del sensor

#define M1Delante 2
#define M1Atras 3

#define M2Delante 4
#define M2Atras 5

#define SensorTrasero 8


TaskHandle_t xd;
SemaphoreHandle_t xSerialSemaphore;

void EncontrarEnemigo( void *pvParameters );
void SensorTras( void *pvParameters );
void SensoresLateral( void *pvParameters );

void MotoresDelante();
void MotoresGirar();
void EsquivarAtras();

void setup() {

  

  pinMode(Trigger, OUTPUT); //pin como salida
  pinMode(Echo, INPUT);  //pin como entrada
  digitalWrite(Trigger, LOW);//Inicializamos el pin con 0

   

  // initialize serial communication at 9600 bits per second:
  Serial.begin(9600);

  pinMode(M1Delante,OUTPUT);
  pinMode(M1Atras,OUTPUT);

  pinMode(M2Delante,OUTPUT);
  pinMode(M2Atras,OUTPUT);

  pinMode(SensorTrasero,INPUT);
  
  while (!Serial) {
    ; // wait for serial port to connect. Needed for native USB, on LEONARDO, MICRO, YUN, and other 32u4 based boards.
  }

  
  if ( xSerialSemaphore == NULL )  // Check to confirm that the Serial Semaphore has not already been created.
  {
    xSerialSemaphore = xSemaphoreCreateMutex();  // Create a mutex semaphore we will use to manage the Serial Port
    if ( ( xSerialSemaphore ) != NULL )
      xSemaphoreGive( ( xSerialSemaphore ) );  // Make the Serial Port available for use, by "Giving" the Semaphore.
  }

  

  // Now set up two Tasks to run independently.
  xTaskCreate(
    EncontrarEnemigo
    ,  "DigitalRead"  // A name just for humans
    ,  128  // This stack size can be checked & adjusted by reading the Stack Highwater
    ,  NULL //Parameters for the task
    ,  2  // Priority, with 3 (configMAX_PRIORITIES - 1) being the highest, and 0 being the lowest.
    ,  &xd ); //Task Handle

  xTaskCreate(
    SensorTras
    ,  "AnalogRead" // A name just for humans
    ,  128  // Stack size
    ,  NULL //Parameters for the task
    ,  0  // Priority
    ,  NULL ); //Task Handle

    xTaskCreate(
    SensoresLateral
    ,  "AnalogRead" // A name just for humans
    ,  128  // Stack size
    ,  NULL //Parameters for the task
    ,  1  // Priority
    ,  NULL ); //Task Handle

}

void loop()
{
  // Empty. Things are done in Tasks.
}

/*--------------------------------------------------*/
/*---------------------- Tasks ---------------------*/
/*--------------------------------------------------*/

void EncontrarEnemigo( void *pvParameters __attribute__((unused)) )  // This is a Task.
{
  /*
    DigitalReadSerial
    Reads a digital input on pin 2, prints the result to the serial monitor

    This example code is in the public domain.
  */

  

  for (;;) // A Task shall never return or exit.
  {
    

    long t; //timepo que demora en llegar el eco
    long d; //distancia en centimetros
  
    digitalWrite(Trigger, HIGH);
    delayMicroseconds(10);          //Enviamos un pulso de 10us
    digitalWrite(Trigger, LOW);
    
    t = pulseIn(Echo, HIGH); //obtenemos el ancho del pulso
    d = t/59;             //escalamos el tiempo a una distancia en cm

    if(d<70)
    {
      MotoresDelante();
    }
    else
    {
      MotoresGirar();
    }

    
    
    if ( xSemaphoreTake( xSerialSemaphore, ( TickType_t ) 5 ) == pdTRUE )
    {
      
     

      xSemaphoreGive( xSerialSemaphore ); // Now free or "Give" the Serial Port for others.
    }

    vTaskDelay(1);  // one tick delay (15ms) in between reads for stability
  }
}

void SensorTras( void *pvParameters __attribute__((unused)) )  // This is a Task.
{


  for (;;)
  {

String estado="Normal";
    if(digitalRead(SensorTrasero))
    {

      EsquivarAtras();
  
      estado="Esquivando";
    }
   
    if ( xSemaphoreTake( xSerialSemaphore, ( TickType_t ) 5 ) == pdTRUE )
    {

      Serial.println(estado);

      xSemaphoreGive( xSerialSemaphore ); // Now free or "Give" the Serial Port for others.
    }

    vTaskDelay(1);  // one tick delay (15ms) in between reads for stability
  }
}

void SensoresLateral( void *pvParameters __attribute__((unused)) )  // This is a Task.
{

  for (;;)
  {
    
    if ( xSemaphoreTake( xSerialSemaphore, ( TickType_t ) 5 ) == pdTRUE )
    {
      

      xSemaphoreGive( xSerialSemaphore ); // Now free or "Give" the Serial Port for others.
    }

    vTaskDelay(1);  // one tick delay (15ms) in between reads for stability
  }
}

void MotoresDelante()
{
  digitalWrite(M1Delante,HIGH);
  digitalWrite(M1Atras,LOW);

  digitalWrite(M2Delante,HIGH);
  digitalWrite(M2Atras,LOW);
  
}

void MotoresGirar()
{
  digitalWrite(M1Delante,LOW);
  digitalWrite(M1Atras,HIGH);

  digitalWrite(M2Delante,HIGH);
  digitalWrite(M2Atras,LOW);
  
}

void EsquivarAtras()
{
  vTaskSuspend( xd );
  digitalWrite(M1Delante,HIGH);
  digitalWrite(M1Atras,LOW);
  
  digitalWrite(M2Delante,LOW);
  digitalWrite(M2Atras,HIGH);

  delay(500);

  digitalWrite(M1Delante,LOW);
  digitalWrite(M1Atras,LOW);

  delay(500);

  vTaskResume( xd );
}
